require 'yaml'
require_relative './heroku'

module BbDeploy
  class Config
    attr_accessor :application_name,
                  :application_urls,
                  :logentries_token,
                  :deployment_channel,
                  :engineering_channel,
                  :slack_webhook_key

    class << self
      def configuration
        @configuration ||= BbDeploy::Config.new
      end

      def configure
        yield(configuration)
        configuration
      end

      def configure_from_yaml(file_path)
        options = YAML.load(ERB.new(File.read(file_path)).result)
        configure do |config|
          %w(application_name application_urls deployment_channel engineering_channel).each do |key|
            config.send("#{key}=", options[key])
          end
        end
      end

      def set_heroku_fields!(phase)
        configure do |config|
          config.logentries_token  = BbDeploy::Heroku.get_variable(phase, 'DEPLOY_LOG_TOKEN').freeze
          config.slack_webhook_key = BbDeploy::Heroku.get_variable(phase, 'SLACK_WEBHOOK').freeze
        end
      end

      def method_missing(method, *args)
        configuration.send(method, *args)
      end
    end
  end
end