# Helper file for Logging/Slack notifications
module BbDeploy::Logger
  def self.logger(phase)
    @logger ||= ConsolidatedLogger::Logentries.new(phase) # Phase-specific memoization
  end

  module ConsolidatedLogger
    # For logging to our Logentries consolidated logging service
    class Logentries
      def initialize(phase)
        @phase = phase
      end

      def with_consolidated_logging
        @original_stdout = $stdout
        # Logging something before assigning $stdout is necessary
        # to avoid `SystemStackError: stack level too deep`?!
        logger.send(:info, "logging #with_consolidated_logging")
        $stdout = self
        yield
      ensure
        $stdout = @original_stdout
      end

      # To replace $stdout we must #write
      def write(msg)
        # Apparently reassigning $stdout triggers a blank message.
        @original_stdout.puts(msg)
        logger.send(:info, "#{ENV['USER']} #{msg}") unless msg.blank?
      end

      def method_missing(method, *args)
        # this is mostly a call to #flush. We do nothing.
      end

      # Logs originating here will be found in the BB
      # Logentries account in the "Ruby Log" for each phase
      def logger
        @logger ||= Le.new(BbDeploy::Config.logentries_token)
      end
    end
  end

  def chat_broadcast(msg, phase:, notify_eng_hub: false)
    attachment = [
      {
        fallback: "Deploying to #{phase}",
        color: " ",
        fields: [
          {
            title: "Details",
            value: msg,
            short: false
          }
        ]
      }
    ]

    slack.ping("*Deploying to #{phase}*", attachments: attachment, channel: BbDeploy::Config.deployment_channel)
    slack.ping("*Deploying to #{phase}*", attachments: attachment, channel: BbDeploy::Config.engineering_channel) if notify_eng_hub
    @consolidated_logger ||= ConsolidatedLogger::Logentries.new(phase)
    @consolidated_logger.logger.info("DEPLOY--#{msg}")
  end

  def slack
    @slack ||= Slack::Notifier.new(BbDeploy::Config.slack_webhook_key)
  end
end